@@ -26,12 +26,14 @@ from django.utils.encoding import python_2_unicode_compatible
 
 import dateutil.tz
 import os
+import os.path
 import datetime
 import pytz
 import shutil
 import filecmp
 
-from spud import media
+from . import media
+from . import managers
 
 SEX_CHOICES = (
     ('1', 'male'),
@@ -40,9 +42,7 @@ SEX_CHOICES = (
 
 PHOTO_ACTION = (
     ('D', 'delete'),
-    ('S', 'regenerate size'),
-    ('R', 'regenerate thumbnails'),
-    ('V', 'regenerate video'),
+    ('R', 'regenerate thumbnails & video'),
     ('M', 'move photo'),
     ('auto', 'rotate automatic'),
     ('90', 'rotate 90 degrees clockwise'),
@@ -51,7 +51,7 @@ PHOTO_ACTION = (
 )
 
 
-class photo_already_exists_error(Exception):
+class PhotoAlreadyExistsError(Exception):
     pass
 
 
@@ -78,7 +78,7 @@ def action_to_string(action):
 
 # BASE ABSTRACT MODEL CLASS
 
-class base_model(models.Model):
+class BaseModel(models.Model):
 
     class Meta:
         abstract = True
@@ -93,15 +93,22 @@ class base_model(models.Model):
         return error_list
 
 
-class hierarchy_model(base_model):
+class HierarchyModel(BaseModel):
 
     class Meta:
         abstract = True
 
     def get_descendants(self, include_self):
+        queryset = self.descendant_set.all()
+        queryset = queryset.prefetch_related(
+            'descendant__cover_photo__photo_thumb_set')
+        queryset = queryset.prefetch_related(
+            'descendant__cover_photo__photo_video_set')
+
         if include_self:
             results = 0
-            for i in self.descendant_set.all():
+
+            for i in queryset:
                 results = results + 1
                 yield i.descendant
             # if descendants list is length 0 it is invalid,
@@ -109,17 +116,28 @@ class hierarchy_model(base_model):
             if results == 0:
                 yield self
         else:
-            for i in self.descendant_set.filter(position__gt=0):
+            for i in queryset.filter(position__gt=0):
                 yield i.descendant
 
     def get_ascendants(self, include_self):
+        queryset = self.ascendant_set.all()
+        queryset = queryset.prefetch_related(
+            'ascendant__cover_photo__photo_thumb_set')
+        queryset = queryset.prefetch_related(
+            'ascendant__cover_photo__photo_video_set')
+
         if include_self:
-            for i in self.ascendant_set.all():
+            for i in queryset.all():
                 yield i.ascendant
         else:
-            for i in self.ascendant_set.filter(position__gt=0):
+            for i in queryset.filter(position__gt=0):
                 yield i.ascendant
 
+        return
+
+    def list_ascendants(self):
+        return self.get_ascendants(False)
+
     def _ascendants_glue(
             self, instance, position, seen, cache, parent_attributes):
         glue = []
@@ -208,22 +226,130 @@ class hierarchy_model(base_model):
 
 
 @python_2_unicode_compatible
-class place(hierarchy_model):
-    place_id = models.AutoField(primary_key=True)
+class album(HierarchyModel):
     parent = models.ForeignKey(
-        'self', related_name='children', null=True, blank=True)
-    title = models.CharField(max_length=192, db_index=True)
-    address = models.CharField(max_length=192, blank=True)
-    address2 = models.CharField(max_length=192, blank=True)
-    city = models.CharField(max_length=96, blank=True)
-    state = models.CharField(max_length=96, blank=True)
-    postcode = models.CharField(max_length=30, blank=True)
-    country = models.CharField(max_length=96, blank=True)
-    url = models.CharField(max_length=3072, blank=True)
-    urldesc = models.CharField(max_length=96, blank=True)
+        'self', related_name='children', null=True, blank=True,
+        on_delete=models.PROTECT)
+    title = models.CharField(
+        max_length=96, null=True, blank=True, db_index=True)
+    description = models.TextField(null=True, blank=True)
     cover_photo = models.ForeignKey(
-        'photo', related_name='place_cover_of', null=True, blank=True)
-    notes = models.TextField(blank=True)
+        'photo', related_name='album_cover_of', null=True, blank=True,
+        on_delete=models.SET_NULL)
+    sort_name = models.CharField(max_length=96, null=True, blank=True)
+    sort_order = models.CharField(max_length=96, null=True, blank=True)
+    revised = models.DateTimeField(null=True, blank=True)
+    revised_utc_offset = models.IntegerField(null=True, blank=True)
+    objects = managers.HierarchyManager()
+
+    class Meta:
+        ordering = ['sort_name', 'sort_order', 'title']
+
+    def __str__(self):
+        return self.title
+
+    def fix_ascendants(self, cache=None, do_descendants=True):
+        self._fix_ascendants(
+            ["parent"], album_ascendant, cache, do_descendants)
+
+    def save(self, *args, **kwargs):
+        super(album, self).save(*args, **kwargs)
+        self.fix_ascendants()
+
+    def check_delete(self):
+        errorlist = []
+        if self.photos.all().count() > 0:
+            errorlist.append("Cannot delete album with photos")
+        if self.children.all().count() > 0:
+            errorlist.append("Cannot delete album with children")
+        return errorlist
+
+    def get_cover_photo(self):
+        cphoto = None
+        if self.cover_photo is not None:
+            cphoto = self.cover_photo
+        else:
+
+            query = photo.objects.filter(albums__ascendant_set__ascendant=self)
+            query = query.exclude(action='D').reverse()
+            try:
+                cphoto = query[0]
+            except IndexError:
+                pass
+        return cphoto
+
+
+@python_2_unicode_compatible
+class category(HierarchyModel):
+    parent = models.ForeignKey(
+        'self', related_name='children', null=True, blank=True,
+        on_delete=models.PROTECT)
+    title = models.CharField(max_length=96, db_index=True)
+    description = models.TextField(null=True, blank=True)
+    cover_photo = models.ForeignKey(
+        'photo', related_name='category_cover_of', null=True, blank=True,
+        on_delete=models.SET_NULL)
+    sort_name = models.CharField(max_length=96, null=True, blank=True)
+    sort_order = models.CharField(max_length=96, null=True, blank=True)
+    objects = managers.HierarchyManager()
+
+    class Meta:
+        ordering = ['sort_name', 'sort_order', 'title']
+
+    def __str__(self):
+        return self.title
+
+    def fix_ascendants(self, cache=None, do_descendants=True):
+        self._fix_ascendants(
+            ["parent"], category_ascendant, cache, do_descendants)
+
+    def save(self, *args, **kwargs):
+        super(category, self).save(*args, **kwargs)
+        self.fix_ascendants()
+
+    def check_delete(self):
+        errorlist = []
+        if self.photos.all().count() > 0:
+            errorlist.append("Cannot delete category with photos")
+        if self.children.all().count() > 0:
+            errorlist.append("Cannot delete category with children")
+        return errorlist
+
+    def get_cover_photo(self):
+        cphoto = None
+        if self.cover_photo is not None:
+            cphoto = self.cover_photo
+        else:
+
+            query = photo.objects.filter(
+                categorys__ascendant_set__ascendant=self)
+            query = query.exclude(action='D').reverse()
+            try:
+                cphoto = query[0]
+            except IndexError:
+                pass
+        return cphoto
+
+
+@python_2_unicode_compatible
+class place(HierarchyModel):
+    parent = models.ForeignKey(
+        'self', related_name='children', null=True, blank=True,
+        on_delete=models.PROTECT)
+    title = models.CharField(max_length=192, db_index=True)
+    address = models.CharField(max_length=192, null=True,  blank=True)
+    address2 = models.CharField(max_length=192, null=True, blank=True)
+    city = models.CharField(max_length=96, null=True, blank=True)
+    state = models.CharField(max_length=96, null=True, blank=True)
+    postcode = models.CharField(max_length=30, null=True, blank=True)
+    country = models.CharField(max_length=96, null=True, blank=True)
+    url = models.CharField(max_length=3072, null=True, blank=True)
+    urldesc = models.CharField(max_length=96, null=True, blank=True)
+    cover_photo = models.ForeignKey(
+        'photo', related_name='place_cover_of', null=True, blank=True,
+        on_delete=models.SET_NULL)
+    notes = models.TextField(null=True, blank=True)
+    objects = managers.HierarchyManager()
 
     class Meta:
         ordering = ['title']
@@ -235,6 +361,10 @@ class place(hierarchy_model):
         self._fix_ascendants(
             ["parent"], place_ascendant, cache, do_descendants)
 
+    def save(self, *args, **kwargs):
+        super(place, self).save(*args, **kwargs)
+        self.fix_ascendants()
+
     def check_delete(self):
         errorlist = []
         if self.photos.all().count() > 0:
@@ -247,144 +377,64 @@ class place(hierarchy_model):
             errorlist.append("Cannot delete home")
         return errorlist
 
-    def delete(self):
-        self.work_of.clear()
-        self.home_of.clear()
-        super(place, self).delete()
-
     def get_cover_photo(self):
-        photo = None
+        cphoto = None
         if self.cover_photo is not None:
-            photo = self.cover_photo
+            cphoto = self.cover_photo
         else:
+
+            query = photo.objects.filter(place__ascendant_set__ascendant=self)
+            query = query.exclude(action='D').reverse()
             try:
-                photo = self.photos.exclude(action='D').reverse()[0]
+                cphoto = query[0]
             except IndexError:
                 pass
-        return photo
+        return cphoto
 
 
 @python_2_unicode_compatible
-class album(hierarchy_model):
-    album_id = models.AutoField(primary_key=True)
-    parent = models.ForeignKey(
-        'self', related_name='children', null=True, blank=True)
-    title = models.CharField(max_length=96, db_index=True)
-    description = models.TextField(blank=True)
-    cover_photo = models.ForeignKey(
-        'photo', related_name='album_cover_of', null=True, blank=True)
-    sort_name = models.CharField(max_length=96, blank=True)
-    sort_order = models.CharField(max_length=96, blank=True)
-    revised = models.DateTimeField(null=True)
-    revised_utc_offset = models.IntegerField(null=True)
-
-    class Meta:
-        ordering = ['sort_name', 'sort_order', 'title']
-
-    def __str__(self):
-        return self.title
-
-    def fix_ascendants(self, cache=None, do_descendants=True):
-        self._fix_ascendants(
-            ["parent"], album_ascendant, cache, do_descendants)
-
-    def check_delete(self):
-        errorlist = []
-        if self.photos.all().count() > 0:
-            errorlist.append("Cannot delete album with photos")
-        if self.children.all().count() > 0:
-            errorlist.append("Cannot delete album with children")
-        return errorlist
-
-    def get_cover_photo(self):
-        photo = None
-        if self.cover_photo is not None:
-            photo = self.cover_photo
-        else:
-            try:
-                photo = self.photos.exclude(action='D').reverse()[0]
-            except IndexError:
-                pass
-        return photo
-
-
-@python_2_unicode_compatible
-class category(hierarchy_model):
-    category_id = models.AutoField(primary_key=True)
-    parent = models.ForeignKey(
-        'self', related_name='children', null=True, blank=True)
-    title = models.CharField(max_length=96, db_index=True)
-    description = models.TextField(blank=True)
-    cover_photo = models.ForeignKey(
-        'photo', related_name='category_cover_of', null=True, blank=True)
-    sort_name = models.CharField(max_length=96, blank=True)
-    sort_order = models.CharField(max_length=96, blank=True)
-
-    class Meta:
-        ordering = ['sort_name', 'sort_order', 'title']
-
-    def __str__(self):
-        return self.title
-
-    def fix_ascendants(self, cache=None, do_descendants=True):
-        self._fix_ascendants(
-            ["parent"], category_ascendant, cache, do_descendants)
-
-    def check_delete(self):
-        errorlist = []
-        if self.photos.all().count() > 0:
-            errorlist.append("Cannot delete category object with photos")
-        if self.children.all().count() > 0:
-            errorlist.append("Cannot delete category object with children")
-        return errorlist
-
-    def get_cover_photo(self):
-        photo = None
-        if self.cover_photo is not None:
-            photo = self.cover_photo
-        else:
-            try:
-                photo = self.photos.exclude(action='D').reverse()[0]
-            except IndexError:
-                pass
-        return photo
-
-
-@python_2_unicode_compatible
-class person(hierarchy_model):
-    person_id = models.AutoField(primary_key=True)
-    first_name = models.CharField(max_length=96, blank=True, db_index=True)
-    last_name = models.CharField(max_length=96, blank=True, db_index=True)
-    middle_name = models.CharField(max_length=96, blank=True)
-    called = models.CharField(max_length=48, blank=True)
-    gender = models.CharField(max_length=1, blank=True, choices=SEX_CHOICES)
+class person(HierarchyModel):
+    first_name = models.CharField(max_length=96, db_index=True)
+    last_name = models.CharField(
+        max_length=96, null=True, blank=True, db_index=True)
+    middle_name = models.CharField(max_length=96, null=True, blank=True)
+    called = models.CharField(max_length=48, null=True, blank=True)
+    sex = models.CharField(
+        max_length=1, null=True, blank=True, choices=SEX_CHOICES)
     dob = models.DateField(null=True, blank=True)
     dod = models.DateField(null=True, blank=True)
     home = models.ForeignKey(
-        place, null=True, blank=True, related_name="home_of")
+        place, null=True, blank=True, related_name="home_of",
+        on_delete=models.SET_NULL)
     work = models.ForeignKey(
-        place, null=True, blank=True, related_name="work_of")
+        place, null=True, blank=True, related_name="work_of",
+        on_delete=models.SET_NULL)
     father = models.ForeignKey(
-        'self', null=True, blank=True, related_name='father_of')
+        'self', null=True, blank=True, related_name='father_of',
+        on_delete=models.SET_NULL)
     mother = models.ForeignKey(
-        'self', null=True, blank=True, related_name='mother_of')
+        'self', null=True, blank=True, related_name='mother_of',
+        on_delete=models.SET_NULL)
     spouse = models.ForeignKey(
-        'self', null=True, blank=True, related_name='reverse_spouses')
-    notes = models.TextField(blank=True)
+        'self', null=True, blank=True, related_name='reverse_spouses',
+        on_delete=models.SET_NULL)
+    notes = models.TextField(null=True, blank=True)
     cover_photo = models.ForeignKey(
-        'photo', related_name='person_cover_of', null=True, blank=True)
-    email = models.CharField(max_length=192, blank=True)
+        'photo', related_name='person_cover_of', null=True, blank=True,
+        on_delete=models.SET_NULL)
+    email = models.CharField(max_length=192, null=True, blank=True)
+    objects = managers.PersonManager()
 
     class Meta:
         ordering = ['last_name', 'first_name']
 
     def __str__(self):
         result = "%s" % (self.first_name)
-        if self.middle_name is not None and self.middle_name != "":
+        if self.middle_name is not None and self.middle_name:
             result += " %s" % (self.middle_name)
-        if self.called is not None and self.called != "":
+        if self.called is not None and self.called:
             result += " (%s)" % (self.called)
-        if self.last_name is not None and self.last_name != "":
+        if self.last_name is not None and self.last_name:
             result += " %s" % (self.last_name)
         return result
 
@@ -402,33 +452,46 @@ class person(hierarchy_model):
             errorlist.append("Cannot delete person that is a photographer")
         return errorlist
 
-    def delete(self):
-        self.father_of.clear()
-        self.mother_of.clear()
-        self.reverse_spouses.clear()
-        self.photographed.clear()
-        super(person, self).delete()
-
     def fix_ascendants(self, cache=None, do_descendants=True):
         self._fix_ascendants(
             ["mother", "father"], person_ascendant, cache, do_descendants)
 
+    def save(self, *args, **kwargs):
+        super(person, self).save(*args, **kwargs)
+        self.fix_ascendants()
+
+    def _queryset(self):
+        queryset = person.objects.all()
+        queryset = queryset.select_related(
+            'cover_photo', 'mother', 'father', 'spouse', 'home', 'work')
+        queryset = queryset.prefetch_related('cover_photo__photo_thumb_set')
+        queryset = queryset.prefetch_related('cover_photo__photo_video_set')
+        return queryset
+
+    # spouses
+
+    def spouses(self):
+        return self._queryset().filter(
+            Q(pk=self.spouse_id) |
+            Q(spouse__id=self.pk)
+        )
+
     # grand parents generation
 
     def grandparents(self):
         parents = self.parents()
-        return person.objects.filter(
-            Q(father_of__in=parents)
-            | Q(mother_of__in=parents))
+        return self._queryset().filter(
+            Q(father_of__in=parents) |
+            Q(mother_of__in=parents))
 
     # parents generation
 
     def uncles_aunts(self):
         parents = [i.pk for i in self.parents()]
         grandparents = self.grandparents()
-        return person.objects.filter(
-            Q(father__in=grandparents)
-            | Q(mother__in=grandparents)).exclude(pk__in=parents)
+        return self._queryset().filter(
+            Q(father__in=grandparents) |
+            Q(mother__in=grandparents)).exclude(pk__in=parents)
 
     def parents(self):
         results = []
@@ -442,29 +505,29 @@ class person(hierarchy_model):
 
     def siblings(self):
         parents = self.parents()
-        return person.objects.filter(
+        return self._queryset().filter(
             Q(father__in=parents) | Q(mother__in=parents)).exclude(pk=self.pk)
 
     def cousins(self):
         parents = self.uncles_aunts()
-        return person.objects.filter(
+        return self._queryset().filter(
             Q(father__in=parents) | Q(mother__in=parents))
 
     # next generation
 
     def children(self):
-        return person.objects.filter(Q(father=self) | Q(mother=self))
+        return self._queryset().filter(Q(father=self) | Q(mother=self))
 
     def nephews_nieces(self):
         parents = self.siblings()
-        return person.objects.filter(
+        return self._queryset().filter(
             Q(father__in=parents) | Q(mother__in=parents))
 
     # grand children generation
 
     def grandchildren(self):
         children = self.children()
-        return person.objects.filter(
+        return self._queryset().filter(
             Q(father__in=children) | Q(mother__in=children))
 
     def get_cover_photo(self):
@@ -479,20 +542,24 @@ class person(hierarchy_model):
         return photo
 
 
-class feedback(hierarchy_model):
-    photo = models.ForeignKey('photo', related_name="feedbacks")
+class feedback(HierarchyModel):
+    cover_photo = models.ForeignKey(
+        'photo', related_name="feedbacks",
+        on_delete=models.CASCADE)
     parent = models.ForeignKey(
-        'self', related_name='children', null=True, blank=True)
+        'self', related_name='children', null=True, blank=True,
+        on_delete=models.CASCADE)
     rating = models.IntegerField()
-    comment = models.TextField(blank=True)
+    comment = models.TextField(null=True, blank=True)
 
     # Information about the user leaving the comment
     user = models.ForeignKey(
         settings.AUTH_USER_MODEL,
-        blank=True, null=True, related_name="photos_feedbacks")
-    user_name = models.CharField(max_length=50, blank=True)
-    user_email = models.EmailField(blank=True)
-    user_url = models.URLField(blank=True)
+        blank=True, null=True, related_name="photos_feedbacks",
+        on_delete=models.SET_NULL)
+    user_name = models.CharField(max_length=50, null=True, blank=True)
+    user_email = models.EmailField(null=True, blank=True)
+    user_url = models.URLField(null=True, blank=True)
 
     # Metadata about the comment
     submit_datetime = models.DateTimeField()
@@ -515,6 +582,7 @@ class feedback(hierarchy_model):
                 .replace(tzinfo=None)
             self.utc_offset = value.utcoffset().seconds / 60
         super(feedback, self).save(*args, **kwargs)
+        self.fix_ascendants()
 
     def fix_ascendants(self, cache=None, do_descendants=True):
         self._fix_ascendants(
@@ -525,38 +593,40 @@ class feedback(hierarchy_model):
 
 
 @python_2_unicode_compatible
-class photo(base_model):
-    photo_id = models.AutoField(primary_key=True)
-    name = models.CharField(max_length=128, blank=True, db_index=True)
-    path = models.CharField(max_length=255, blank=True, db_index=True)
+class photo(BaseModel):
+    name = models.CharField(max_length=128, db_index=True)
+    path = models.CharField(max_length=255, db_index=True)
     size = models.IntegerField(null=True, blank=True)
-    title = models.CharField(max_length=64, blank=True, db_index=True)
+    title = models.CharField(
+        max_length=64, null=True, blank=True, db_index=True)
     photographer = models.ForeignKey(
-        person, null=True, blank=True, related_name='photographed')
-    location = models.ForeignKey(
-        place, null=True, blank=True, related_name='photos')
-    view = models.CharField(max_length=64, blank=True)
+        person, null=True, blank=True, related_name='photographed',
+        on_delete=models.SET_NULL)
+    place = models.ForeignKey(
+        place, null=True, blank=True, related_name='photos',
+        on_delete=models.SET_NULL)
+    view = models.CharField(max_length=64, null=True, blank=True)
     rating = models.FloatField(null=True, blank=True, db_index=True)
-    description = models.TextField(blank=True)
+    description = models.TextField(null=True, blank=True)
     utc_offset = models.IntegerField()
     datetime = models.DateTimeField(db_index=True)
-    camera_make = models.CharField(max_length=32, blank=True)
-    camera_model = models.CharField(max_length=32, blank=True)
-    flash_used = models.CharField(max_length=1, blank=True)
-    focal_length = models.CharField(max_length=64, blank=True)
-    exposure = models.CharField(max_length=64, blank=True)
-    compression = models.CharField(max_length=64, blank=True)
-    aperture = models.CharField(max_length=16, blank=True)
+    camera_make = models.CharField(max_length=32, null=True, blank=True)
+    camera_model = models.CharField(max_length=32, null=True, blank=True)
+    flash_used = models.CharField(max_length=1, null=True, blank=True)
+    focal_length = models.CharField(max_length=64, null=True, blank=True)
+    exposure = models.CharField(max_length=64, null=True, blank=True)
+    compression = models.CharField(max_length=64, null=True, blank=True)
+    aperture = models.CharField(max_length=16, null=True, blank=True)
     level = models.IntegerField()
-    iso_equiv = models.CharField(max_length=8, blank=True)
-    metering_mode = models.CharField(max_length=32, blank=True)
-    focus_dist = models.CharField(max_length=16, blank=True)
-    ccd_width = models.CharField(max_length=16, blank=True)
-    comment = models.TextField(blank=True)
+    iso_equiv = models.CharField(max_length=8, null=True, blank=True)
+    metering_mode = models.CharField(max_length=32, null=True, blank=True)
+    focus_dist = models.CharField(max_length=16, null=True, blank=True)
+    ccd_width = models.CharField(max_length=16, null=True, blank=True)
+    comment = models.TextField(null=True, blank=True)
     action = models.CharField(
         max_length=4, null=True, blank=True,
         choices=PHOTO_ACTION, db_index=True)
-    timestamp = models.DateTimeField()
+    timestamp = models.DateTimeField(auto_now_add=True)
 
     albums = models.ManyToManyField(
         album, through='photo_album', related_name='photos')
@@ -567,8 +637,10 @@ class photo(base_model):
     relations = models.ManyToManyField(
         'self', through='photo_relation', symmetrical=False)
 
+    objects = managers.PhotoManager()
+
     class Meta:
-        ordering = ['datetime', 'photo_id']
+        ordering = ['datetime', 'id']
 
     def __str__(self):
         if self.title is None or self.title == "":
@@ -576,6 +648,10 @@ class photo(base_model):
         else:
                 return self.title
 
+    @property
+    def timezone(self):
+        return pytz.FixedOffset(self.utc_offset)
+
     def fix_rating(self):
         feedbacks = self.feedbacks.filter(is_removed=False)
         if len(feedbacks) == 0:
@@ -598,8 +674,9 @@ class photo(base_model):
     def _get_thumb_path(cls, size, path, name):
         if size in settings.IMAGE_SIZES:
             (shortname, _) = os.path.splitext(name)
-            return "%sthumb/%s/%s/%s.jpg" % (
-                settings.IMAGE_PATH, size, path, shortname)
+            return os.path.join(
+                settings.IMAGE_PATH, "thumb",
+                size, path, shortname + ".jpg")
         else:
             raise RuntimeError("unknown image size %s" % (size))
 
@@ -607,14 +684,15 @@ class photo(base_model):
     def _get_video_path(cls, size, path, name, extension):
         if size in settings.VIDEO_SIZES:
             (shortname, _) = os.path.splitext(name)
-            return "%svideo/%s/%s/%s.%s" % (
-                settings.IMAGE_PATH, size, path, shortname, extension)
+            return os.path.join(
+                settings.IMAGE_PATH, "video",
+                size, path, shortname + "." + extension)
         else:
             raise RuntimeError("unknown image size %s" % (size))
 
     @classmethod
     def _get_orig_path(cls, path, name):
-        return "%sorig/%s/%s" % (settings.IMAGE_PATH, path, name)
+        return os.path.join(settings.IMAGE_PATH, "orig", path, name)
 
     # Public methods
 
@@ -622,8 +700,9 @@ class photo(base_model):
         return self._get_orig_path(self.path, self.name)
 
     def get_orig_url(self):
-        return iri_to_uri("%sorig/%s/%s" % (
-            settings.IMAGE_URL, urlquote(self.path), urlquote(self.name)))
+        return iri_to_uri(os.path.join(
+            settings.IMAGE_URL, "orig",
+            urlquote(self.path), urlquote(self.name)))
 
     def get_thumb(self, size):
         try:
@@ -631,23 +710,28 @@ class photo(base_model):
         except photo_thumb.DoesNotExist:
             return None
 
+    def get_thumbs(self):
+        return list(self.photo_thumb_set.all())
+
     def get_video(self, size, format):
         try:
             return self.photo_video_set.get(size=size, format=format)
         except photo_thumb.DoesNotExist:
             return None
 
+    def get_videos(self):
+        return list(self.photo_video_set.all())
+
     # Other stuff
     def check_delete(self):
         errorlist = []
         return errorlist
 
     def delete(self):
-        self.place_cover_of.clear()
-        self.album_cover_of.clear()
-        self.category_cover_of.clear()
-        self.person_cover_of.clear()
-        os.unlink(self.get_orig_path())
+        if self.name:
+            path = self.get_orig_path()
+            if os.path.lexists(path):
+                os.unlink(path)
         for pt in self.photo_thumb_set.all():
             path = pt.get_path()
             if os.path.lexists(path):
@@ -664,12 +748,10 @@ class photo(base_model):
         m.rotate(amount)
 
         (width, height) = m.get_size()
-        size = m.get_bytes()
+        self.size = os.path.getsize(self.get_orig_path())
 
-        # FIXME
         self.width = width
         self.height = height
-        self.size = size
         return
     rotate.alters_data = True
 
@@ -677,7 +759,7 @@ class photo(base_model):
         m = media.get_media(self.get_orig_path())
         umask = os.umask(0o022)
 
-        for size, s in settings.IMAGE_SIZES.iteritems():
+        for size, s in settings.IMAGE_SIZES.items():
             dst = self._get_thumb_path(size, self.path, self.name)
             if not os.path.lexists(os.path.dirname(dst)):
                 os.makedirs(os.path.dirname(dst), 0o755)
@@ -700,8 +782,8 @@ class photo(base_model):
         umask = os.umask(0o022)
 
         if m.is_video():
-            for size, s in settings.VIDEO_SIZES.iteritems():
-                for format, f in settings.VIDEO_FORMATS.iteritems():
+            for size, s in settings.VIDEO_SIZES.items():
+                for format, f in settings.VIDEO_FORMATS.items():
                     dst = self._get_video_path(
                         size, self.path, self.name, f['extension'])
                     if not os.path.lexists(os.path.dirname(dst)):
@@ -741,102 +823,6 @@ class photo(base_model):
         return
     update_size.alters_data = True
 
-    def update_from_source(self, media=None):
-        if media is None:
-            m = media.get_media(self.get_orig_path())
-        else:
-            m = media
-
-        exif = m.get_exif()
-
-        (width, height) = m.get_size()
-        size = m.get_bytes()
-
-        # FIXME
-        self.width = width
-        self.height = height
-        self.size = size
-
-        try:
-            self.camera_make = exif['EXIF:Make']
-        except KeyError:
-            pass
-
-        try:
-            self.camera_model = exif['EXIF:Model']
-        except KeyError:
-            pass
-
-        try:
-            if int(exif['EXIF:Flash']) & 1:
-                self.flash_used = 'Y'
-            else:
-                self.flash_used = 'N'
-        except KeyError:
-            pass
-
-        try:
-            self.focal_length = exif['EXIF:FocalLength']
-        except KeyError:
-            pass
-
-        try:
-            self.exposure = exif['EXIF:ExposureTime']
-        except KeyError:
-            pass
-
-        try:
-            self.compression = ""
-        except KeyError:
-            pass
-
-        try:
-            fnumber = exif['EXIF:FNumber']
-            self.aperture = "F%.1f" % (fnumber)
-        except KeyError:
-            pass
-
-        try:
-            self.iso_equiv = exif['EXIF:ISO']
-        except KeyError:
-            pass
-
-        try:
-            value = int(exif['EXIF:MeteringMode'])
-            if value == 0:
-                self.metering_mode = "unknown"
-            elif value == 1:
-                self.metering_mode = "average"
-            elif value == 2:
-                self.metering_mode = "center weighted average"
-            elif value == 3:
-                self.metering_mode = "spot"
-            elif value == 4:
-                self.metering_mode = "multi spot"
-            elif value == 5:
-                self.metering_mode = "pattern"
-            elif value == 6:
-                self.metering_mode = "partial"
-            elif value == 255:
-                self.metering_mode = "other"
-            else:
-                self.metering_mode = "reserved"
-        except KeyError:
-            pass
-
-        try:
-            self.focus_dist = exif['Composite:HyperfocalDistance']
-        except KeyError:
-            pass
-
-        try:
-            self.ccd_width = ""
-        except KeyError:
-            pass
-
-        return
-    update_from_source.alters_data = True
-
     @classmethod
     def get_conflicts(cls, new_path, new_name):
         # check for conflicts or errors
@@ -878,7 +864,7 @@ class photo(base_model):
 
             dupfile = dups[0].get_orig_path()
             if filecmp.cmp(old_file, dupfile):
-                raise photo_already_exists_error(
+                raise PhotoAlreadyExistsError(
                     "same photo %d already exists at %s/%s as %s/%s" %
                     (dups[0].pk, new_path, new_name,
                         dups[0].path, dups[0].name))
@@ -985,8 +971,8 @@ class photo(base_model):
 # ---------------------------------------------------------------------------
 
 
-class photo_thumb(base_model):
-    photo = models.ForeignKey(photo)
+class photo_thumb(BaseModel):
+    photo = models.ForeignKey(photo, on_delete=models.CASCADE)
     size = models.CharField(max_length=10, db_index=True)
     width = models.IntegerField(null=True, blank=True)
     height = models.IntegerField(null=True, blank=True)
@@ -994,16 +980,16 @@ class photo_thumb(base_model):
     def get_path(self):
         photo = self.photo
         (shortname, _) = os.path.splitext(photo.name)
-        return "%sthumb/%s/%s/%s.jpg" % (
-            settings.IMAGE_PATH, self.size,
-            photo.path, shortname)
+        return os.path.join(
+            settings.IMAGE_PATH, "thumb", self.size,
+            photo.path, shortname + ".jpg")
 
     def get_url(self):
         photo = self.photo
         (shortname, _) = os.path.splitext(photo.name)
-        return iri_to_uri("%sthumb/%s/%s/%s.jpg" % (
-            settings.IMAGE_URL, urlquote(self.size),
-            urlquote(photo.path), urlquote(shortname)))
+        return iri_to_uri(os.path.join(
+            settings.IMAGE_URL, "thumb", urlquote(self.size),
+            urlquote(photo.path), urlquote(shortname + ".jpg")))
 
     def delete(self):
         path = self.get_path()
@@ -1011,8 +997,8 @@ class photo_thumb(base_model):
             os.unlink(path)
 
 
-class photo_video(base_model):
-    photo = models.ForeignKey(photo)
+class photo_video(BaseModel):
+    photo = models.ForeignKey(photo, on_delete=models.CASCADE)
     size = models.CharField(max_length=10, db_index=True)
     width = models.IntegerField(null=True, blank=True)
     height = models.IntegerField(null=True, blank=True)
@@ -1022,17 +1008,16 @@ class photo_video(base_model):
     def get_path(self):
         photo = self.photo
         (shortname, _) = os.path.splitext(photo.name)
-        return "%svideo/%s/%s/%s.%s" % (
-            settings.IMAGE_PATH, self.size,
-            photo.path, shortname, self.extension)
+        return os.path.join(
+            settings.IMAGE_PATH, "video", self.size,
+            photo.path, shortname + "." + self.extension)
 
     def get_url(self):
         photo = self.photo
         (shortname, _) = os.path.splitext(photo.name)
-        return iri_to_uri("%svideo/%s/%s/%s.%s" % (
-            settings.IMAGE_URL, urlquote(self.size),
-            urlquote(photo.path), urlquote(shortname),
-            urlquote(self.extension)))
+        return iri_to_uri(os.path.join(
+            settings.IMAGE_URL, "video", urlquote(self.size),
+            urlquote(photo.path), urlquote(shortname + "." + self.extension)))
 
     def delete(self):
         path = self.get_path()
@@ -1040,19 +1025,19 @@ class photo_video(base_model):
             os.unlink(path)
 
 
-class photo_album(base_model):
-    photo = models.ForeignKey(photo)
-    album = models.ForeignKey(album)
+class photo_album(BaseModel):
+    photo = models.ForeignKey(photo, on_delete=models.CASCADE)
+    album = models.ForeignKey(album, on_delete=models.CASCADE)
 
 
-class photo_category(base_model):
-    photo = models.ForeignKey(photo)
-    category = models.ForeignKey(category)
+class photo_category(BaseModel):
+    photo = models.ForeignKey(photo, on_delete=models.CASCADE)
+    category = models.ForeignKey(category, on_delete=models.CASCADE)
 
 
-class photo_person(base_model):
-    photo = models.ForeignKey(photo)
-    person = models.ForeignKey(person)
+class photo_person(BaseModel):
+    photo = models.ForeignKey(photo, on_delete=models.CASCADE)
+    person = models.ForeignKey(person, on_delete=models.CASCADE)
     position = models.IntegerField(null=True, blank=True)
 
     class Meta:
@@ -1060,14 +1045,19 @@ class photo_person(base_model):
 
 
 @python_2_unicode_compatible
-class photo_relation(base_model):
+class photo_relation(BaseModel):
     photo_1 = models.ForeignKey(
-        photo, db_column="photo_id_1", related_name="relations_1")
+        photo, db_column="photo_id_1", related_name="relations_1",
+        on_delete=models.CASCADE)
     photo_2 = models.ForeignKey(
-        photo, db_column="photo_id_2", related_name="relations_2")
+        photo, db_column="photo_id_2", related_name="relations_2",
+        on_delete=models.CASCADE)
     desc_1 = models.CharField(max_length=384)
     desc_2 = models.CharField(max_length=384)
 
+    class Meta:
+        unique_together = ('photo_1', 'photo_2')
+
     def __str__(self):
         return "relationship '%s' to '%s'" % (self.photo_1, self.photo_2)
 
@@ -1075,7 +1065,7 @@ class photo_relation(base_model):
 # ---------------------------------------------------------------------------
 
 
-class album_ascendant(base_model):
+class album_ascendant(BaseModel):
     ascendant = models.ForeignKey(album, related_name='descendant_set')
     descendant = models.ForeignKey(album, related_name='ascendant_set')
     position = models.IntegerField()
@@ -1084,7 +1074,7 @@ class album_ascendant(base_model):
         ordering = ['position']
 
 
-class category_ascendant(base_model):
+class category_ascendant(BaseModel):
     ascendant = models.ForeignKey(category, related_name='descendant_set')
     descendant = models.ForeignKey(category, related_name='ascendant_set')
     position = models.IntegerField()
@@ -1093,7 +1083,7 @@ class category_ascendant(base_model):
         ordering = ['position']
 
 
-class place_ascendant(base_model):
+class place_ascendant(BaseModel):
     ascendant = models.ForeignKey(place, related_name='descendant_set')
     descendant = models.ForeignKey(place, related_name='ascendant_set')
     position = models.IntegerField()
@@ -1102,7 +1092,7 @@ class place_ascendant(base_model):
         ordering = ['position']
 
 
-class person_ascendant(base_model):
+class person_ascendant(BaseModel):
     ascendant = models.ForeignKey(person, related_name='descendant_set')
     descendant = models.ForeignKey(person, related_name='ascendant_set')
     position = models.IntegerField()
@@ -1111,7 +1101,7 @@ class person_ascendant(base_model):
         ordering = ['position']
 
 
-class feedback_ascendant(base_model):
+class feedback_ascendant(BaseModel):
     ascendant = models.ForeignKey(feedback, related_name='descendant_set')
     descendant = models.ForeignKey(feedback, related_name='ascendant_set')
     position = models.IntegerField()
